// src/checkbox.ts
import { createProvideScope } from "@oku-ui/provide";
import { Transition, computed, defineComponent, h, onMounted, ref, toRefs, watchEffect } from "vue";
import { composeEventHandlers } from "@oku-ui/utils";
import { useControllable, usePrevious, useRef, useSize } from "@oku-ui/use-composable";
import { Primitive } from "@oku-ui/primitive";
function isIndeterminate(checked) {
  return checked === "indeterminate";
}
function getState(checked) {
  return isIndeterminate(checked) ? "indeterminate" : checked ? "checked" : "unchecked";
}
var BubbleInput = defineComponent({
  name: "BubbleInput",
  inheritAttrs: false,
  props: {
    checked: {
      type: [Boolean, String, Number],
      default: false
    },
    control: {
      type: Object,
      default: null
    },
    bubbles: {
      type: Boolean,
      default: true
    }
  },
  setup(props, { attrs }) {
    const { ...inputAttrs } = attrs;
    const { checked, bubbles, control } = toRefs(props);
    const _ref = ref();
    const prevChecked = usePrevious(checked);
    const controlSize = useSize(control);
    watchEffect(() => {
      const input = _ref.value;
      const inputProto = window.HTMLInputElement.prototype;
      const descriptor = Object.getOwnPropertyDescriptor(inputProto, "checked");
      const setChecked = descriptor.set;
      if (prevChecked !== checked.value && setChecked) {
        const event = new Event("click", { bubbles: bubbles.value });
        input.indeterminate = isIndeterminate(checked.value);
        setChecked.call(input, isIndeterminate(checked.value) ? false : checked);
        input.dispatchEvent(event);
      }
    });
    return () => h("input", {
      "type": "checkbox",
      "aria-hidden": true,
      "defaultChecked": isIndeterminate(checked.value) ? false : checked,
      ...inputAttrs,
      "tabIndex": -1,
      "ref": _ref,
      "style": {
        ...inputAttrs.style,
        ...controlSize,
        position: "absolute",
        pointerEvents: "none",
        opacity: 0,
        margin: 0
      }
    });
  }
});
var CHECKBOX_NAME = "Checkbox";
var [createCheckboxProvider, _createCheckboxScope] = createProvideScope(CHECKBOX_NAME);
var [CheckboxProvider, useCheckboxInject] = createCheckboxProvider(CHECKBOX_NAME);
var checkboxDisplayName = "OkuCheckbox";
var Checkbox = defineComponent({
  name: checkboxDisplayName,
  components: { BubbleInput },
  inheritAttrs: false,
  props: {
    modelValue: {
      type: [Boolean, String, Number],
      default: void 0
    },
    checked: {
      type: [Boolean, String, Number],
      default: void 0
    },
    defaultChecked: {
      type: [Boolean, String],
      default: void 0
    },
    required: {
      type: Boolean,
      default: void 0
    },
    scopeCheckbox: {
      type: Object,
      required: false,
      default: void 0
    }
  },
  emits: ["update:checked", "update:modelValue"],
  setup(props, { attrs, slots, expose, emit }) {
    const { checked: checkedProp, scopeCheckbox, defaultChecked, required } = toRefs(props);
    const { newRef, $el } = useRef();
    expose({
      innerRef: $el
    });
    const {
      name,
      disabled,
      value = "on",
      ...checkboxProps
    } = attrs;
    const hasConsumerStoppedPropagationRef = ref(false);
    const isFormControl = ref(false);
    const { state, updateValue } = useControllable({
      prop: computed(() => checkedProp.value),
      defaultProp: computed(() => defaultChecked.value),
      onChange: (result) => {
        emit("update:checked", result);
        emit("update:modelValue", result);
      }
    });
    const initialCheckedStateRef = ref();
    onMounted(() => {
      isFormControl.value = Boolean($el.value.closest("form")) || false;
      initialCheckedStateRef.value = state.value;
    });
    watchEffect(() => {
      var _a;
      const form = (_a = newRef.value) == null ? void 0 : _a.$el.form;
      if (form) {
        const reset = () => updateValue(initialCheckedStateRef.value);
        form.addEventListener("reset", reset);
        return () => form.removeEventListener("reset", reset);
      }
    });
    CheckboxProvider({
      scope: scopeCheckbox.value,
      state,
      disabled
    });
    const originalReturn = () => [
      h(
        Primitive.button,
        {
          "type": "button",
          "role": "checkbox",
          "aria-checked": isIndeterminate(state.value) ? "mixed" : state.value,
          "aria-required": required.value,
          "data-state": getState(state.value),
          "data-disabled": disabled ? "" : void 0,
          "disabled": disabled,
          "value": value,
          ...checkboxProps,
          "ref": newRef,
          "onKeyDown": composeEventHandlers(checkboxProps.onKeydown, (event) => {
            if (event.key === "Enter")
              event.preventDefault();
          }),
          "onClick": composeEventHandlers(checkboxProps.onClick, (event) => {
            const data = isIndeterminate(checkedProp.value) ? true : !checkedProp.value;
            if (state.value === data)
              updateValue(!data);
            else if (state.value === "indeterminate")
              updateValue(!data);
            else
              updateValue(data);
            if (isFormControl.value) {
              if (!hasConsumerStoppedPropagationRef.value)
                event.stopPropagation();
            }
          })
        },
        {
          default: () => {
            var _a;
            return (_a = slots.default) == null ? void 0 : _a.call(slots);
          }
        }
      ),
      isFormControl.value && h(
        BubbleInput,
        {
          bubbles: !hasConsumerStoppedPropagationRef.value,
          name,
          value,
          checked: state.value,
          required,
          disabled,
          // We transform because the input is absolutely positioned but we have
          // rendered it **after** the button. This pulls it back to sit on top
          // of the button.
          style: { transform: "translateX(-100%)" }
        }
      )
    ];
    return originalReturn;
  }
});
var INDICATOR_NAME = "CheckboxIndicator";
var CheckboxIndicator = defineComponent({
  name: "CheckboxIndicator",
  components: { Transition },
  props: {
    scopeCheckbox: {
      type: Object,
      required: false
    },
    forceMount: Boolean
  },
  setup(props, { attrs, expose, slots }) {
    const { scopeCheckbox, forceMount } = toRefs(props);
    const { ...indicatorProps } = attrs;
    const { $el, newRef } = useRef();
    expose({
      innerRef: $el
    });
    const context = useCheckboxInject(INDICATOR_NAME, scopeCheckbox.value);
    const originalReturn = () => h(Transition, {}, {
      default: () => forceMount.value || isIndeterminate(context.value.state.value) || context.value.state.value === true ? h(
        Primitive.span,
        {
          "ref": newRef,
          "data-state": getState(context.value.state.value),
          "data-disabled": context.value.disabled ? "" : void 0,
          ...indicatorProps,
          "style": { pointerEvents: "none", ...attrs.style }
        },
        {
          default: () => {
            var _a;
            return (_a = slots.default) == null ? void 0 : _a.call(slots);
          }
        }
      ) : null
    });
    return originalReturn;
  }
});
var OkuCheckbox = Checkbox;
var OkuCheckboxIndicator = CheckboxIndicator;
export {
  OkuCheckbox,
  OkuCheckboxIndicator
};
